/* ==========================================================================
        video.h
   --------------------------------------------------------------------------
        Copyright (C) 1987,1988,1989.  All Rights Reserved.
        Daniel J. Reynolds
        High Aspect Development Corporation
        9821 South Pulaski Road
        Evergreen Park, IL  60642   312-424-5997
   ========================================================================== */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/* -------------------------
    make things easier defs
   ------------------------- */
#define uint    unsigned int
#define ushort  unsigned short int
#define ulong   unsigned long int
#define uchar   unsigned char
#define TRUE    (1)                         /* required for curses */
#define FALSE   (0)                         /* required for curses */
#define OK      (1)                         /* required for curses */
#define ERR     (0)                         /* required for curses */
#define ON      (TRUE)                      /* used for d_break() */
#define OFF     (FALSE)                     /* used for d_break() */
/* ---------------------
    terminal mode flags
   --------------------- */
#define VIDEO_RAM       (0x0001)
#define IBM_BIOS        (0x0002)
#define ANSI_SYS        (0x0004)
#define XANSI_SYS       (0x0008)
/* -----------------
    standard colors
   ----------------- */
#define c_BLACK         (0x0000)
#define c_BLUE          (0x0100)
#define c_GREEN         (0x0200)
#define c_CYAN          (0x0300)
#define c_RED           (0x0400)
#define c_MAGENTA       (0x0500)
#define c_BROWN         (0x0600)
#define c_WHITE         (0x0700)
/* -------------
    video modes
   ------------- */
#define vm_BW40         (0)
#define vm_CO40         (1)
#define vm_BW80         (2)
#define vm_CO80         (3)
#define vm_MONO         (7)
/* ----------------
    video adapters
   ---------------- */
#define va_NONE         (0)
#define va_MDA          (1)
#define va_CGA          (2)
#define va_EGA          (3)
#define va_MCGA         (4)
#define va_VGA          (5)
#define va_HGC          (0x80)
#define va_HGCPlus      (0x81)
#define va_InColor      (0x82)
/* ----------------
    video displays
   ---------------- */
#define vd_NONE         (0)
#define vd_MDA          (1)
#define vd_CGA          (2)
#define vd_EGA          (3)
#define vd_PS2Mono      (4)
#define vd_PS2Color     (5)
/* -----------------------
    default video systems
   ----------------------- */
#define vs_IBMPC_M      (11)
#define vs_IBMPC_C      (22)
#define vs_IBMPC_EM     (31)
#define vs_IBMPC_EC     (32)
#define vs_IBMPC_EE     (33)
#define vs_IBMPC_MM     (44)
#define vs_IBMPC_MC     (45)
#define vs_IBMPC_VM     (54)
#define vs_IBMPC_VC     (55)
/* ---------------------
    box character types
   --------------------- */
#define BOX_NONE        (0)
#define BOX_GENERIC     (1)
#define BOX_SINGLE      (2)
#define BOX_DOUBLE      (3)
#define BOX_DHORZ       (4)
#define BOX_DVERT       (5)
#define BOX_BLOCK       (6)
#define BOX_LSHADE      (7)
#define BOX_MSHADE      (8)
#define BOX_HSHADE      (9)
/* -------------------
    video ram buffers
   ------------------- */
#define COLO_BUFF       (0xb800)
#define MONO_BUFF       (0xb000)
/* ------------------------
    cursor start/end lines
   ------------------------ */
#define cs_MN           (0x0b0c)
#define cs_MB           (0x000d)
#define cs_CN           (0x0607)
#define cs_CB           (0x0007)
/* ------------------------------------------------
    manifest constants required by the ansi driver
   ------------------------------------------------ */
#define ANSI_NORMAL     (0)
#define ANSI_BOLD       (1)
#define ANSI_UNDERLINE  (4)
#define ANSI_BLINK      (5)
#define ANSI_REVERSE    (7)
#define ANSI_INVISIBLE  (8)
#define ANSI_BLACK      (0)
#define ANSI_RED        (1)
#define ANSI_GREEN      (2)
#define ANSI_YELLOW     (3)
#define ANSI_BLUE       (4)
#define ANSI_MAGENTA    (5)
#define ANSI_CYAN       (6)
#define ANSI_WHITE      (7)
#define ANSI_BW40       (0)
#define ANSI_C40        (1)
#define ANSI_BW80       (2)
#define ANSI_C80        (3)
#define ANSI_C320       (4)
#define ANSI_BW320      (5)
#define ANSI_BW640      (6)
#define ANSI_WRAP       (7)
#define ANSI_FOREGROUND (30)
#define ANSI_BACKGROUND (40)
/* --------------------------------------------------------------------------
    The following manifest contants define the entire set of possible
    keystrokes in the IBM PC environment.  The values for the standard
    ASCII characters are simple the ASCII codes assigned to the character.
    The numbers assigned to the function keys, alternate keys, and
    cursor control keys are the negative of the IBM scan code for that key.
   -------------------------------------------------------------------------- */
#define CTL_A           1
#define CTL_B           2
#define CTL_C           3
#define CTL_D           4
#define CTL_E           5
#define CTL_F           6
#define CTL_G           7
#define CTL_H           8
#define CTL_I           9
#define CTL_J           10
#define CTL_K           11
#define CTL_L           12
#define CTL_M           13
#define CTL_N           14
#define CTL_O           15
#define CTL_P           16
#define CTL_Q           17
#define CTL_R           18
#define CTL_S           19
#define CTL_T           20
#define CTL_U           21
#define CTL_V           22
#define CTL_W           23
#define CTL_X           24
#define CTL_Y           25
#define CTL_Z           26
#define ESCAPE          27
#define SPACE           32
#define EX_POINT        33
#define DOUBLE_QUOTE    34
#define POUNDSIGN       35
#define DOLLARSIGN      36
#define PCTSIGN         37
#define AMPERSAND       38
#define SINGLE_QUOTE    39
#define LT_PAREN        40
#define RT_PAREN        41
#define ASTERISK        42
#define PLUSSIGN        43
#define COMMA           44
#define DASH            45
#define PERIOD          46
#define SLASH           47
#define ZERO            48
#define ONE             49
#define TWO             50
#define THREE           51
#define FOUR            52
#define FIVE            53
#define SIX             54
#define SEVEN           55
#define EIGHT           56
#define NINE            57
#define COLON           58
#define SEMICOLON       59
#define LESS_THAN       60
#define EQUAL           61
#define GREATER_THAN    62
#define QUESTION        63
#define ATSIGN          64
#define UP_A            65
#define UP_B            66
#define UP_C            67
#define UP_D            68
#define UP_E            69
#define UP_F            70
#define UP_G            71
#define UP_H            72
#define UP_I            73
#define UP_J            74
#define UP_K            75
#define UP_L            76
#define UP_M            77
#define UP_N            78
#define UP_O            79
#define UP_P            80
#define UP_Q            81
#define UP_R            82
#define UP_S            83
#define UP_T            84
#define UP_U            85
#define UP_V            86
#define UP_W            87
#define UP_X            88
#define UP_Y            89
#define UP_Z            90
#define LT_BRACKET      91
#define BACKSLASH       92
#define RT_BRACKET      93
#define CAROT           94
#define UNDERSCORE      95
#define RT_QUOTE        96
#define LOW_a           97
#define LOW_b           98
#define LOW_c           99
#define LOW_d           100
#define LOW_e           101
#define LOW_f           102
#define LOW_g           103
#define LOW_h           104
#define LOW_i           105
#define LOW_j           106
#define LOW_k           107
#define LOW_l           108
#define LOW_m           109
#define LOW_n           110
#define LOW_o           111
#define LOW_p           112
#define LOW_q           113
#define LOW_r           114
#define LOW_s           115
#define LOW_t           116
#define LOW_u           117
#define LOW_v           118
#define LOW_w           119
#define LOW_x           120
#define LOW_y           121
#define LOW_z           122
#define LT_BRACE        123
#define OR              124
#define RT_BRACE        125
#define TILDE           126
#define CTL_BACKSP      127
#define BELL            CTL_G
#define BACKSPACE       CTL_H
#define TAB             CTL_I
#define LINEFEED        CTL_J
#define NEWLINE         CTL_J
#define FORMFEED        CTL_L
#define ENTER           CTL_M
#define CTL_ENTER       CTL_J
#define COMPRES_ON      CTL_O
#define COMPRES_OFF     CTL_R
#define REVTAB          -15
#define ALT_Q           -16
#define ALT_W           -17
#define ALT_E           -18
#define ALT_R           -19
#define ALT_T           -20
#define ALT_Y           -21
#define ALT_U           -22
#define ALT_I           -23
#define ALT_O           -24
#define ALT_P           -25
#define ALT_A           -30
#define ALT_S           -31
#define ALT_D           -32
#define ALT_F           -33
#define ALT_G           -34
#define ALT_H           -35
#define ALT_J           -36
#define ALT_K           -37
#define ALT_L           -38
#define ALT_Z           -44
#define ALT_X           -45
#define ALT_C           -46
#define ALT_V           -47
#define ALT_B           -48
#define ALT_N           -49
#define ALT_M           -50
#define F1              -59
#define F2              -60
#define F3              -61
#define F4              -62
#define F5              -63
#define F6              -64
#define F7              -65
#define F8              -66
#define F9              -67
#define F10             -68
#define HOME            -71
#define UP_ARROW        -72
#define PAGE_UP         -73
#define LT_ARROW        -75
#define RT_ARROW        -77
#define END             -79
#define DOWN_ARROW      -80
#define PAGE_DOWN       -81
#define INSERT          -82
#define DELETE          -83
#define SFT_F1          -84
#define SFT_F2          -85
#define SFT_F3          -86
#define SFT_F4          -87
#define SFT_F5          -88
#define SFT_F6          -89
#define SFT_F7          -90
#define SFT_F8          -91
#define SFT_F9          -92
#define SFT_F10         -93
#define CTL_F1          -94
#define CTL_F2          -95
#define CTL_F3          -96
#define CTL_F4          -97
#define CTL_F5          -98
#define CTL_F6          -99
#define CTL_F7          -100
#define CTL_F8          -101
#define CTL_F9          -102
#define CTL_F10         -103
#define ALT_F1          -104
#define ALT_F2          -105
#define ALT_F3          -106
#define ALT_F4          -107
#define ALT_F5          -108
#define ALT_F6          -109
#define ALT_F7          -110
#define ALT_F8          -111
#define ALT_F9          -112
#define ALT_F10         -113
#define CTL_PRTSC       -114
#define CTL_LT_ARROW    -115
#define CTL_RT_ARROW    -116
#define CTL_END         -117
#define CTL_PAGE_DOWN   -118
#define CTL_HOME        -119
#define ALT_1           -120
#define ALT_2           -121
#define ALT_3           -122
#define ALT_4           -123
#define ALT_5           -124
#define ALT_6           -125
#define ALT_7           -126
#define ALT_8           -127
#define ALT_9           -128
#define ALT_0           -129
#define ALT_MINUS       -130
#define ALT_EQUAL       -131
#define CTL_PAGE_UP     -132
/* ---------------------------------------
    macro definitions for video functions
   --------------------------------------- */
#define attr(f,b)       ((((b) << 4) + (f)) & 0xff00)
#define revr_attr(a)    ((((a) << 4) + ((a) >> 4)) & 0xff00)
#define set_ram()       (cur_video->v_cfg->Tmode = VIDEO_RAM)
#define set_bios()      (cur_video->v_cfg->Tmode = IBM_BIOS)
#define set_ansi()      (cur_video->v_cfg->Tmode = ANSI_SYS)
#define set_xansi()     (cur_video->v_cfg->Tmode = XANSI_SYS)
#define is_ram()        (cur_video->v_cfg->Tmode == VIDEO_RAM)
#define is_bios()       (cur_video->v_cfg->Tmode == IBM_BIOS)
#define is_ansi()       (cur_video->v_cfg->Tmode == ANSI_SYS)
#define is_xansi()      (cur_video->v_cfg->Tmode == XANSI_SYS)
#define v_setcur(v)     (cur_video = v)
/* --------------------------------------------------------------------------
        video structures
   -------------------------------------------------------------------------- */
typedef struct {            /* MAINTAINS INSTALLED VIDEO CONFIGURATION  */
    char a_adapt;           /* active video adapter */
    char a_display;         /* active video display type */
    char i_adapt;           /* installed but inactive video adapter */
    char i_display;         /* installed but inactive video diaply type */
} V_SYS;
typedef struct {            /* MAINTAINS PREVIOUS VIDEO INFORMATION */
    int  sav_mode;          /* users previous video mode */
    int  sav_curs;          /* users previous cursor type */
    int  sav_page;          /* users previous video page */
    int  sav_cpos;          /* users previous cursor position */
} V_SAV;
typedef struct {            /* MAINTAINS CURRENT VIDEO INFORMATION */
    char Vname[16];         /* Video system name */
    int  Vmode;             /* current video mode */
    int  Tmode;             /* current termninal mode */
    int  Vpage;             /* video page */
    int  buff;              /* video memory address */
    int  snow;              /* CGA snow routine flag */
    int  Nattr;             /* normal attribute */
    int  Uattr;             /* underline attribute */
    int  Ncursor;           /* normal cursor */
    int  Bcursor;           /* block cursor */
    int  Icursor;           /* invisible cursor */
} V_CFG;
typedef struct {
    V_SYS * v_sys;          /* Installed video hardware */
    V_SAV * v_prog;         /* Previous video configuration */
    V_SAV * v_shell;        /* Previous video configuration */
    V_CFG * v_cfg;          /* Current video configuration */
    void  * v_user;         /* Pointer to user defined data */
} VIDEO;
/* ------------------------------
    Line drawing types structure
   ------------------------------ */
typedef struct {            /* CONTAINS THE IBM LINE DRAWING CHARACTERS */
    uint h;                 /* horizontal character */
    uint v;                 /* vertical character */
    uint c;                 /* center character */
    uint lt;                /* left tee */
    uint rt;                /* right tee */
    uint tt;                /* top tee */
    uint bt;                /* bottom tee */
    uint ur;                /* upper right corner */
    uint ul;                /* upper left corner */
    uint lr;                /* lower right corner */
    uint ll;                /* lower left corner */
} BOX;
/* -----------------------
    external declerations
   ----------------------- */
extern int break_flag;      /* flags CTL-C and CTL-BREAK */
extern int critr_flag;      /* critical error flag */
extern int caltd_flag;      /* CTL-ALT-DELETE flag */
extern int attr_trans;      /* attribute translate */
extern V_CFG v_terms[];     /* default video terminal array */
extern int N_TERMS;         /* number of video terminals defined */
extern VIDEO * cur_video;   /* current video structure */
extern BOX   box_typ[];     /* array of all IBM PC box types */
extern int N_BOXES;         /* number of boxtypes defined */
/* ---------------------------
    IBM bios curser functions
   --------------------------- */
extern int  c_getpos(void);
extern int  c_setpos(int);
extern int  c_getrow(void);
extern int  c_getcol(void);
extern int  c_gshape(void);
extern void c_sshape(int);
extern void c_set(int,int);
/* ---------------------------------
    80x86 specific memory functions
   --------------------------------- */
extern void m_move(char *,char *,int);
extern void m_fillb(char *,int,int);
extern void m_fillw(char *,int,int);
extern int  m_peekb(int,int);
extern int  m_pokeb(int,int,int);
extern int  m_peekw(int,int);
extern int  m_pokew(int,int,int);
extern int  m_exist(int,int);
extern uint m_avail(void);
/* ------------------------------------------
    IBM bios specific video & misc functions
   ------------------------------------------ */
extern VIDEO * v_init(char *,int *);
extern V_CFG * v_getdef(char *);
extern char * v_default(void);
extern void v_free(VIDEO *);
extern int  v_setdef(char *,V_CFG *);
extern void v_adapt(V_SYS *);
extern void v_egarow(int);
extern int  v_seterm(char *);
extern void v_save(V_SAV *);
extern void v_restore(V_SAV *);
extern void v_setvid(V_CFG *);
extern void v_setmod(int);
extern int  v_getmod(void);
extern int  v_getpag(void);
extern void v_setpag(int);
extern int  v_getkey(void);
extern int  v_sound(int,int);
extern void v_capon(void);
extern void v_capoff(void);
extern int  v_iskey(void);
extern void v_clrscr(void);
extern int  v_screen(int *,int);
extern void a_tputca(int,int,int);
extern void v_toscrn(char *,int,int);
extern void v_putc(int);
extern void v_putca(int);
extern void v_nosnow(char *);
extern void a_trans(int *,int);
extern int  v_trans(int);
extern void a_settab(char *,char *,char *);
/* --------------------
    ANSI sys functions
   -------------------- */
extern void ansi_cub(int);
extern void ansi_cud(int);
extern void ansi_cuf(int);
extern void ansi_cup(int,int);
extern void ansi_cuu(int);
extern void ansi_ed(void);
extern void ansi_el(void);
extern void ansi_rcp(void);
extern void ansi_scp(void);
extern void ansi_sgr(int);
extern void ansi_rm(int);
extern void ansi_sm(int);
extern void ansi_hvp(int,int);
extern void ansi_cpr(int,int);
extern void ansi_dsr(int *,int *);
/* -------------------------------
    MS-DOS video & misc functions
   ------------------------------- */
extern void d_putc(int);
extern int  d_getkey(void);
extern int  d_iskey(void);
extern void d_sleep(int);
extern long d_ticks(void);
extern void d_break(int);
extern void d_reboot(void);
/* ----------------------------
    Hardware control functions
   ---------------------------- */
extern void break_on(int *);
extern void break_off(void);
extern void criter_on(int *);
extern void criter_off(void);
extern void caltd_on(void);
extern void caltd_off(int *);

